/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.smoke.tools;

// $Id: ScriptTestCase.java 5022 2007-11-09 11:18:34Z thomas.diesler@jboss.com $


import junit.framework.TestCase;

import java.io.File;
import java.io.IOException;

/**
 * [JBWS-1793] Provide a test case for the tools scripts that reside under JBOSS_HOME/bin
 * 
 * http://jira.jboss.org/jira/browse/JBWS-1793
 * 
 * Test the wsprovide and wsconsume scripts that reside
 * under JBOSS_HOME/bin. This basically verifies all dependencies are
 * met to run the shell scripts.
 * 
 * @author Heiko.Braun@jboss.com
 */
public class ScriptTestCase extends TestCase
{    
   private String WSDL_LOCATION = "resources/jaxws/smoke/tools/wsdl/TestServiceCatalog.wsdl";
   private String ENDPOINT_CLASS = "org.jboss.test.ws.jaxws.smoke.tools.CalculatorBean";

   private String JBOSS_HOME;
   private String JDK_HOME;
   private String TEST_EXEC_DIR;
   private String OS;

   protected void setUp() throws Exception
   {
      super.setUp();

      JBOSS_HOME = System.getProperty("jboss.home");
      TEST_EXEC_DIR = System.getProperty("test.execution.dir");
      JDK_HOME = System.getProperty("jdk.home");
      OS = System.getProperty("os.name").toLowerCase();
   }

   public void testWSConsumeFromCommandLine() throws Exception
   {
      if(isWindowsOS())
      {
         fail("This test has not been verified on windows");
      }

      // use absolute path for the output to be re-usable
      String absWsdlLoc= new File(WSDL_LOCATION).getAbsolutePath();
      String absOutput = new File("wsconsume/java").getAbsolutePath();

      String command = JBOSS_HOME + "/bin/wsconsume.sh -k -o "+absOutput+" "+ absWsdlLoc;
      Process p = executeCommand(command);

      // check status code
      assertStatusCode(p, "wsconsume");

      File javaSource = new File("wsconsume/java/org/openuri/_2004/_04/helloworld/EndpointInterface.java");
     
      assertTrue("Service endpoint interface not generated", javaSource.exists());
   }

   public void testWSProvideFromCommandLine() throws Exception
   {
      if(isWindowsOS())
      {
         fail("This test has not been verified on windows");
      }

      String absOutput = new File("wsprovide/java/").getAbsolutePath();

      String command = JBOSS_HOME + "/bin/wsprovide.sh -k -w -o "+absOutput+" --classpath "+TEST_EXEC_DIR+"/classes "+ ENDPOINT_CLASS;
      Process p = executeCommand(command);

      // check status code
      assertStatusCode(p, "wsprovide");

      File outputDir = new File("wsprovide/java");
      File javaSource = new File(
        outputDir.getAbsolutePath()+
          "/org/jboss/test/ws/jaxws/smoke/tools/jaxws/AddResponse.java"
        );
      
      assertTrue("Response wrapper not generated", javaSource.exists());
   }

   private Process executeCommand(String command)
     throws IOException
   {
      // be verbose      
      System.out.println("cmd: " + command);
      System.out.println("test execution dir: " + TEST_EXEC_DIR);

      Process p = Runtime.getRuntime().exec(
        command,
        new String[] {"JBOSS_HOME="+ JBOSS_HOME, "JAVA_HOME="+ JDK_HOME}
      );
      return p;
   }
   
   private void assertStatusCode(Process p, String s)
     throws InterruptedException
   {
      // check status code
      int status = p.waitFor();
      assertTrue(s +" did exit with status " + status, status==0);
   }

   private boolean isWindowsOS()
   {
      return ( (OS.indexOf("nt") > -1) || (OS.indexOf("windows") > -1 ));        
   }
}
